﻿using System;
using System.Net;
using System.Xml;
using System.Text;


namespace Actions
{
    public static class BingTranslationActions
    {
        const string AppId = "E6229FE0535A7ED00FAD685A301D0A767FBD2821";

        static public string TranslationRequest(string from, string to, string word)
        {
            HttpWebRequest request = BuildRequest(from, to, word);
            try
            {
                return DisplayResponse((HttpWebResponse)request.GetResponse());
            }
            catch (WebException ex)
            {
                //todo: handle exception
            }
            throw new Exception(string.Format("There was an error processing the translation request of '{0}' from {1} to {2}", word, from, to));
        }

        static HttpWebRequest BuildRequest(string from, string to, string word)
        {
            string requestString = "http://api.bing.net/xml.aspx?"

                // Common request fields (required)
                + "AppId=" + AppId
                + "&Query="+word
                + "&Sources=Translation"

                // Common request fields (optional)
                + "&Version=2.2"

                // SourceType-specific request fields (required)
                + "&Translation.SourceLanguage="+from
                + "&Translation.TargetLanguage="+to;

            // Create and initialize the request.
            HttpWebRequest request = (HttpWebRequest)HttpWebRequest.Create(
                requestString);

            return request;
        }

        static string DisplayResponse(HttpWebResponse response)
        {
            // Load the response into an XmlDocument.
            XmlDocument document = new XmlDocument();
            document.Load(response.GetResponseStream());

            // Add the default namespace to the namespace manager.
            XmlNamespaceManager nsmgr = new XmlNamespaceManager(
                document.NameTable);
            nsmgr.AddNamespace(
                "api",
                "http://schemas.microsoft.com/LiveSearch/2008/04/XML/element");

            XmlNodeList errors = document.DocumentElement.SelectNodes(
                "./api:Errors/api:Error",
                nsmgr);

            if (errors.Count > 0)
            {
                // There are errors in the response. Display error details.
                return DisplayErrors(errors);
            }
            else
            {
                // There were no errors in the response. Display the
                // Translation results.
                return DisplayResults(document.DocumentElement, nsmgr);
            }
        }

        static string DisplayResults(XmlNode root, XmlNamespaceManager nsmgr)
        {
            StringBuilder str = new StringBuilder();
            str.AppendLine("Bing API Version: " + root.SelectSingleNode("./@Version", nsmgr).InnerText);
            str.AppendLine("Search terms: " + root.SelectSingleNode(
                "./api:Query/api:SearchTerms",
                nsmgr).InnerText);
            str.AppendLine();
            // Add the Translation SourceType namespace to the namespace manager.
            nsmgr.AddNamespace(
                "tra",
                "http://schemas.microsoft.com/LiveSearch/2008/04/XML/translation");

            XmlNodeList results = root.SelectNodes(
                "./tra:Translation/tra:Results/tra:TranslationResult",
                nsmgr);

            // Display the Translation results.
            foreach (XmlNode result in results)
            {
                str.AppendLine(result.SelectSingleNode("./tra:TranslatedTerm",
                nsmgr).InnerText);
            }
            return str.ToString();
        }
        static string DisplayErrors(XmlNodeList errors)
        {
            StringBuilder str = new StringBuilder();
            // Iterate over the list of errors and display error details.
            str.AppendLine("The following errors occurred while trying to translate:");
            str.AppendLine();
            foreach (XmlNode error in errors)
            {
                foreach (XmlNode detail in error.ChildNodes)
                {
                    str.AppendLine(detail.Name + ": " + detail.InnerText);
                }
            }
            return str.ToString();
        }

    }
}
