﻿using System;
using System.Net;
using System.Xml;
using System.Text;
using System.Linq;
using System.Xml.Linq;

namespace Actions
{
    public static class NewsActions
    {
        const string AppId = "E6229FE0535A7ED00FAD685A301D0A767FBD2821";

        static public string GetTheNews()
        {
            HttpWebRequest request = BuildRequest();
            try
            {
                return DisplayResponse((HttpWebResponse)request.GetResponse());
            }
            catch (WebException ex)
            {
                //todo: handle exception
            }
            throw new Exception("There was an error processing the news request.");
        }

        static HttpWebRequest BuildRequest()
        {
            string requestString = "http://api.bing.net/xml.aspx?"

                // Common request fields (required)
            + "AppId=" + AppId
            + "&Query="//msn moneycentral"
            + "&Sources=News"

            // Common request fields (optional)
            + "&Version=2.0"
            + "&Market=en-us"
            + "&Options=EnableHighlighting"

            // News-specific request fields (optional)
            + "&News.Offset=0"

            // The following request fields are mutually exclusive.
                // Uncomment the line corresponding to the request field you wish
                // to use.
                //+ "&News.LocationOverride=US.VA"
                ////+ "&News.Category=rt_Political";
            + "&News.SortBy=Relevance";



            // Create and initialize the request.
            HttpWebRequest request = (HttpWebRequest)HttpWebRequest.Create(
                requestString);

            return request;
        }

        static string DisplayResponse(HttpWebResponse response)
        {
            // Load the response into an XmlDocument.
            XmlDocument document = new XmlDocument();
            document.Load(response.GetResponseStream());

            // Add the default namespace to the namespace manager.
            XmlNamespaceManager nsmgr = new XmlNamespaceManager(
                document.NameTable);
            nsmgr.AddNamespace(
                "api",
                "http://schemas.microsoft.com/LiveSearch/2008/04/XML/element");

            XmlNodeList errors = document.DocumentElement.SelectNodes(
                "./api:Errors/api:Error",
                nsmgr);

            if (errors.Count > 0)
            {
                // There are errors in the response. Display error details.
                return DisplayErrors(errors);
            }
            else
            {
                // There were no errors in the response. Display the
                // News results.
                return DisplayResults(document.DocumentElement, nsmgr);
            }
        }

        static string DisplayResults(XmlNode root, XmlNamespaceManager nsmgr)
        {
            //XDocument doc = XDocument.Load(new XmlNodeReader(root));
            //XNamespace newsnmsp = "http://schemas.microsoft.com/LiveSearch/2008/04/XML/news";
            //var stories = from newsArt in doc.Elements(newsnmsp + "NewsArticle")
            //              select
            StringBuilder str = new StringBuilder();
            // Add the News SourceType namespace to the namespace manager.
            nsmgr.AddNamespace(
                "news",
                "http://schemas.microsoft.com/LiveSearch/2008/04/XML/news");

            XmlNode news = root.SelectSingleNode("./news:News", nsmgr);
            XmlNodeList results = news.SelectNodes(
                "./news:Results/news:NewsResult",
                nsmgr);

            string version = root.SelectSingleNode("./@Version", nsmgr).InnerText;
            string searchTerms = root.SelectSingleNode(
                "./api:Query/api:SearchTerms",
                nsmgr).InnerText;
            str.AppendLine("Bing API Version " + version);
            str.AppendLine("News results for " + searchTerms);
            str.AppendLine();

            // Display the News results.
            System.Text.StringBuilder builder = new System.Text.StringBuilder();
            foreach (XmlNode result in results)
            {
                builder.AppendLine(
                    result.SelectSingleNode("./news:Title", nsmgr).InnerText);
                builder.AppendLine(
                    result.SelectSingleNode("./news:Url", nsmgr).InnerText);
                builder.AppendLine(
                    result.SelectSingleNode("./news:Source", nsmgr).InnerText);
            }
            str.AppendLine(builder.ToString());
            return str.ToString();
        }

        static string DisplayErrors(XmlNodeList errors)
        {
            StringBuilder str = new StringBuilder();
            // Iterate over the list of errors and display error details.
            str.AppendLine("The following errors were encountered while trying to get the headline news:");
            str.AppendLine();
            foreach (XmlNode error in errors)
            {
                foreach (XmlNode detail in error.ChildNodes)
                {
                    str.AppendLine(detail.Name + ": " + detail.InnerText);
                }

                str.AppendLine();
            }
            return str.ToString();
        }

    }
}
