﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;

namespace LilyNUI
{
    public enum PendingAction
    {
        NoneOutstanding = 0,
        PlayAudio,
        PlayVideo,
        Powerpoint,
        Word,
        Excel,
        ProjectPlan,
        Undefined
    }

    public enum ButtonSelected
    {
        NoButton = 0,
        Button1,
        Button2,
        Button3,
        Button4,
        Button5,
        Button6,
        Button7,
        Button8
    }
    public class ButtonActionEvaluator : INotifyPropertyChanged
    {
        private static ButtonActionEvaluator _me = new ButtonActionEvaluator();

        public static ButtonActionEvaluator GetButtonActionEvaluator()
        {
            if (_me == null)
                _me = new ButtonActionEvaluator();
            return _me;
        }

        private ButtonActionEvaluator()
        {
            actionPending = PendingAction.NoneOutstanding;
            selectedButton = ButtonSelected.NoButton;
        }

        private PendingAction actionPending = PendingAction.NoneOutstanding;
        public PendingAction ActionPending
        {
            get { return actionPending; }
            set
            {
                actionPending = value;
                NotifyPropertyChanged("ActionPending");
            }
        }

        private ButtonSelected selectedButton = ButtonSelected.NoButton;
        public ButtonSelected SelectedButton
        {
            get { return selectedButton; }
            set
            {
                selectedButton = value;
                NotifyPropertyChanged("SelectedButton");
            }
        }

        /// <summary>
        /// This method is used to get an integer representation of the Button/Action state - used in conjection with a switch statement to determine appropriate action.
        /// </summary>
        /// <returns>[int] representing the state of the Button/Action context.</returns>
        public int EvaluateCriteria()
        {
            if (actionPending == PendingAction.NoneOutstanding || selectedButton == ButtonSelected.NoButton)
                return 0;
            return Convert.ToInt16((ConvertToBin((int)selectedButton) + ConvertToBin((int)actionPending)), 2);
        }

        static string ConvertToBin(int i)
        {
            string r = Convert.ToString(i, 2);
            while (r.Length < 4)
                r = "0" + r;
            return r;
        }

        private void NotifyPropertyChanged(string info)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(info));
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;
    }
}
