﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Actions;

namespace LilyNUI
{
    /// <summary>
    /// This class implements the Singleton design pattern (in thread-safe manner) to allow only one LilyContext to exist.
    /// The LilyContext is useful in understanding the context that the system is a part of.
    /// </summary>
    public class LilyContext
    {
        #region Structs

        /// <summary>
        /// This struct is used to contain the data relevant to tracking changing context by the SpeechTracker.
        /// </summary>
        public struct ContextEntry
        {
            public SpeechTracker.Context context;
            public DateTime timestamp;
        };

        #endregion

        #region Class member variables and Properties
        private ButtonActionEvaluator buttonActionEvaluator;
        public ButtonActionEvaluator ButtonActionEvaluator
        {
            get { return buttonActionEvaluator; }
            set { buttonActionEvaluator = value; }
        }


        private bool isMoviePlaying = false;
        public bool IsMoviePlaying
        {
            get { return isMoviePlaying; }
            set { isMoviePlaying = value; }
        }


        private bool isMusicPlaying = false;
        public bool IsMusicPlaying
        {
            get { return isMusicPlaying; }
            set { isMusicPlaying = value; }
        }



        private Person endUser;
        public Person EndUser
        {
            get { return endUser; }
            set { endUser = value; }
        }

        private SpeechTracker.Context currentContext;
        public SpeechTracker.Context CurrentContext
        {
            get { return currentContext; }
        }

        private List<ContextEntry> contextHistory;
        public List<ContextEntry> ContextHistory
        {
            get { return contextHistory; }
        }


        private SpeechTracker sp_trk;
        public SpeechTracker GetSpeechTracker
        {
            get { return sp_trk; }
        }
        static private LilyContext _me = new LilyContext();

        #endregion

        #region Constructors


        private LilyContext()
        {
            sp_trk = new SpeechTracker();
            sp_trk.SpeechContextChanged += new EventHandler<SpeechTracker.SpeechContextChangedArgs>(sp_trk_SpeechContextChanged);
            endUser = new Person();
            currentContext = SpeechTracker.Context.General;
            contextHistory = new List<ContextEntry>();
            contextHistory.Add(new ContextEntry() { context = SpeechTracker.Context.General, timestamp = DateTime.Now });
            buttonActionEvaluator = ButtonActionEvaluator.GetButtonActionEvaluator();
            buttonActionEvaluator.PropertyChanged += new System.ComponentModel.PropertyChangedEventHandler(buttonActionEvaluator_PropertyChanged);
        }

                
        #endregion

        #region Private Methods

        void buttonActionEvaluator_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            if (MultiModalReactions.ActOnMultiModalInput(buttonActionEvaluator.EvaluateCriteria()) == PendingActionResult.ClearPendingAction)
                buttonActionEvaluator.ActionPending = PendingAction.NoneOutstanding;
        }

        void sp_trk_SpeechContextChanged(object sender, SpeechTracker.SpeechContextChangedArgs e)
        {
            contextHistory.Add(new ContextEntry() { context = e.context, timestamp = DateTime.Now });
            currentContext = e.context;
        }

        #endregion


        #region Public Methods

        public static LilyContext GetLilyContext()
        {
            if (_me == null)
                _me = new LilyContext();
            return _me;
        }


        #endregion
    }
}
