﻿using System;
using System.Resources;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;
using Actions;
using System.Threading;
using LilyNUI;
using Microsoft.Research.Kinect.Nui;
//Phidgets is the rfid reader that I used in the project for GMU - removed from code because most will not already have 
//using Phidgets;
//using Phidgets.Events;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Reflection;
using Coding4Fun.Kinect.Wpf;
using Coding4Fun.Kinect.Wpf.Controls;
using System.Net;
using System.Xml;
namespace LilyWPF
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        #region Class Member Variables

        private SpeechTracker sp_tkr;
        private Runtime nui;
        //private RFID rfid;
        private string currentRFIDTag;
        private Queue<string> rfidHistory;
        private Dictionary<string, Person> knownPeople;
        private bool nuiInitialized = false;
        private string[] pulseResponses;
        private LilyContext _context = LilyContext.GetLilyContext();
        private List<Button> buttons;
        private static bool _isClosing = false;
        private static double _handLeft;
        private static double _handTop;
        private static Button _selectedButton = null;
        private bool trackHands = false;
        private CameraControl cameraControl = CameraControl.Course;

        int totalFrames = 0;
        int lastFrames = 0;
        DateTime lastTime = DateTime.MaxValue;

        // We want to control how depth data gets converted into false-color data
        // for more intuitive visualization, so we keep 32-bit color frame buffer versions of
        // these, to be updated whenever we receive and process a 16-bit frame.
        const int RED_IDX = 2;
        const int GREEN_IDX = 1;
        const int BLUE_IDX = 0;
        byte[] depthFrame32 = new byte[320 * 240 * 4];


        Dictionary<JointID, Brush> jointColors = new Dictionary<JointID, Brush>() { 
            {JointID.HipCenter, new SolidColorBrush(Color.FromRgb(169, 176, 155))},
            {JointID.Spine, new SolidColorBrush(Color.FromRgb(169, 176, 155))},
            {JointID.ShoulderCenter, new SolidColorBrush(Color.FromRgb(168, 230, 29))},
            {JointID.Head, new SolidColorBrush(Color.FromRgb(200, 0,   0))},
            {JointID.ShoulderLeft, new SolidColorBrush(Color.FromRgb(79,  84,  33))},
            {JointID.ElbowLeft, new SolidColorBrush(Color.FromRgb(84,  33,  42))},
            {JointID.WristLeft, new SolidColorBrush(Color.FromRgb(255, 126, 0))},
            {JointID.HandLeft, new SolidColorBrush(Color.FromRgb(215,  86, 0))},
            {JointID.ShoulderRight, new SolidColorBrush(Color.FromRgb(33,  79,  84))},
            {JointID.ElbowRight, new SolidColorBrush(Color.FromRgb(33,  33,  84))},
            {JointID.WristRight, new SolidColorBrush(Color.FromRgb(77,  109, 243))},
            {JointID.HandRight, new SolidColorBrush(Color.FromRgb(37,   69, 243))},
            {JointID.HipLeft, new SolidColorBrush(Color.FromRgb(77,  109, 243))},
            {JointID.KneeLeft, new SolidColorBrush(Color.FromRgb(69,  33,  84))},
            {JointID.AnkleLeft, new SolidColorBrush(Color.FromRgb(229, 170, 122))},
            {JointID.FootLeft, new SolidColorBrush(Color.FromRgb(255, 126, 0))},
            {JointID.HipRight, new SolidColorBrush(Color.FromRgb(181, 165, 213))},
            {JointID.KneeRight, new SolidColorBrush(Color.FromRgb(71, 222,  76))},
            {JointID.AnkleRight, new SolidColorBrush(Color.FromRgb(245, 228, 156))},
            {JointID.FootRight, new SolidColorBrush(Color.FromRgb(77,  109, 243))}
        };

        public int ContextHistoryCount
        {
            get { return _context.ContextHistory.Count; }
        }

        #endregion

        #region Enumerations

        enum CameraControl
        {
            Fine,
            Course
        }

        #endregion

        #region Constructors

        public MainWindow()
        {
            InitializeComponent();
            InitializeButtons();
            pulseResponses = new string[] { "Yes?",
                                            "Yes, {0}?",
                                            "I am listening.",
                                            "I am listening, {0}",
                                            "I am ready.",
                                            "I am ready, {0}",
                                            "I am here.",
                                            "How can I help you?",
                                            "{0}, how may I be of service?",
                                            "Can I do something?",
                                            "Do you need something?",
                                            "Do you need something, {0}?" };
            rfidHistory = new Queue<string>(100);//keep a history of the last 100 rfid tags read by the system
            GeneralActions.SpeakAsync("Starting up my components.");
            _context.EndUser = new Person("110033e117", Person.Gender.Male, "Leland Holmquest", "Leland");
            InitializeKinect();
            nui.VideoFrameReady += new EventHandler<ImageFrameReadyEventArgs>(nui_VideoFrameReady);
            nui.SkeletonFrameReady += new EventHandler<SkeletonFrameReadyEventArgs>(nui_SkeletonFrameReady);
            nui.DepthFrameReady += new EventHandler<ImageFrameReadyEventArgs>(nui_DepthFrameReady);
            kinectButton.Click += new RoutedEventHandler(kinectButton_Click);
            sp_tkr = _context.GetSpeechTracker;
            sp_tkr.SpeechDetected += new EventHandler<SpeechTracker.IntentionArgs>(sp_tkr_SpeechDetected);
            sp_tkr.SpeechContextChanged += new EventHandler<SpeechTracker.SpeechContextChangedArgs>(sp_tkr_SpeechContextChanged);
            button1.Click += new RoutedEventHandler(button1_Click);
            button2.Click += new RoutedEventHandler(button2_Click);
            button3.Click += new RoutedEventHandler(button3_Click);
            button4.Click += new RoutedEventHandler(button4_Click);
            button5.Click += new RoutedEventHandler(button5_Click);
            button6.Click += new RoutedEventHandler(button6_Click);
            button7.Click += new RoutedEventHandler(button7_Click);
            button8.Click += new RoutedEventHandler(button8_Click);
            txtBlkContextHistCnt.Text = _context.ContextHistory.Count.ToString();
            //InitializeRFID();

            //load up the dictionary with known users
            // this block not used without an rfid reader
            //LoadRfidAccounts();
        }
                      

        #endregion

        #region Private Methods
        
        void kinectButton_Click(object sender, RoutedEventArgs e)
        {
            _selectedButton.RaiseEvent(new RoutedEventArgs(ButtonBase.ClickEvent, _selectedButton));
        }

        private Point getDisplayPosition(Joint joint)
        {
            float depthX, depthY;
            nui.SkeletonEngine.SkeletonToDepthImage(joint.Position, out depthX, out depthY);
            depthX = Math.Max(0, Math.Min(depthX * 320, 320));  //convert to 320, 240 space
            depthY = Math.Max(0, Math.Min(depthY * 240, 240));  //convert to 320, 240 space
            int colorX, colorY;
            ImageViewArea iv = new ImageViewArea();
            // only ImageResolution.Resolution640x480 is supported at this point
            nui.NuiCamera.GetColorPixelCoordinatesFromDepthPixel(ImageResolution.Resolution640x480, iv, (int)depthX, (int)depthY, (short)0, out colorX, out colorY);

            // map back to skeleton.Width & skeleton.Height
            return new Point((int)(skeletonImage.Width * colorX / 640.0), (int)(skeletonImage.Height * colorY / 480));
        }

        Polyline getBodySegment(Microsoft.Research.Kinect.Nui.JointsCollection joints, Brush brush, params JointID[] ids)
        {
            PointCollection points = new PointCollection(ids.Length);
            for (int i = 0; i < ids.Length; ++i)
            {
                points.Add(getDisplayPosition(joints[ids[i]]));
            }

            Polyline polyline = new Polyline();
            polyline.Points = points;
            polyline.Stroke = brush;
            polyline.StrokeThickness = 5;
            return polyline;
        }

        void renderSkeleton(object sender, SkeletonFrameReadyEventArgs e)
        {
            SkeletonFrame skeletonFrame = e.SkeletonFrame;
            int iSkeleton = 0;
            Brush[] brushes = new Brush[6];
            brushes[0] = new SolidColorBrush(Color.FromRgb(255, 0, 0));
            brushes[1] = new SolidColorBrush(Color.FromRgb(0, 255, 0));
            brushes[2] = new SolidColorBrush(Color.FromRgb(64, 255, 255));
            brushes[3] = new SolidColorBrush(Color.FromRgb(255, 255, 64));
            brushes[4] = new SolidColorBrush(Color.FromRgb(255, 64, 255));
            brushes[5] = new SolidColorBrush(Color.FromRgb(128, 128, 255));

            skeletonImage.Children.Clear();
            foreach (SkeletonData data in skeletonFrame.Skeletons)
            {
                if (SkeletonTrackingState.Tracked == data.TrackingState)
                {
                    // Draw bones
                    Brush brush = brushes[iSkeleton % brushes.Length];
                    skeletonImage.Children.Add(getBodySegment(data.Joints, brush, JointID.HipCenter, JointID.Spine, JointID.ShoulderCenter, JointID.Head));
                    skeletonImage.Children.Add(getBodySegment(data.Joints, brush, JointID.ShoulderCenter, JointID.ShoulderLeft, JointID.ElbowLeft, JointID.WristLeft, JointID.HandLeft));
                    skeletonImage.Children.Add(getBodySegment(data.Joints, brush, JointID.ShoulderCenter, JointID.ShoulderRight, JointID.ElbowRight, JointID.WristRight, JointID.HandRight));
                    skeletonImage.Children.Add(getBodySegment(data.Joints, brush, JointID.HipCenter, JointID.HipLeft, JointID.KneeLeft, JointID.AnkleLeft, JointID.FootLeft));
                    skeletonImage.Children.Add(getBodySegment(data.Joints, brush, JointID.HipCenter, JointID.HipRight, JointID.KneeRight, JointID.AnkleRight, JointID.FootRight));

                    // Draw joints
                    foreach (Joint joint in data.Joints)
                    {
                        Point jointPos = getDisplayPosition(joint);
                        Line jointLine = new Line();
                        jointLine.X1 = jointPos.X - 3;
                        jointLine.X2 = jointLine.X1 + 6;
                        jointLine.Y1 = jointLine.Y2 = jointPos.Y;
                        jointLine.Stroke = jointColors[joint.ID];
                        jointLine.StrokeThickness = 6;
                        skeletonImage.Children.Add(jointLine);
                    }
                }
                iSkeleton++;
            } // for each skeleton
        }

        void nui_SkeletonFrameReady(object sender, SkeletonFrameReadyEventArgs e)
        {
            renderSkeleton(sender, e);
            if (!trackHands)
                return;// if the user deosn't want to use the buttons, simply return
            if (e.SkeletonFrame.Skeletons.Count() == 0)
                return;//no skeletons => don't bother processing
            SkeletonFrame skeletonSet = e.SkeletonFrame;
            SkeletonData firstPerson = (from s in skeletonSet.Skeletons
                                        where s.TrackingState == SkeletonTrackingState.Tracked
                                        orderby s.UserIndex descending
                                        select s).FirstOrDefault();
            if (firstPerson == null)
                return;//if no one is being tracked, no sense in continueing

            JointsCollection joints = firstPerson.Joints;
            Joint righthand = joints[JointID.HandRight];
            Joint lefthand = joints[JointID.HandLeft];
            //use the heighth of the hand to figure out which is being used
            Joint joinCursorHand = (righthand.Position.Y > lefthand.Position.Y)
                ? righthand
                : lefthand;

            float posX = joinCursorHand.ScaleTo((int)SystemParameters.PrimaryScreenWidth, (int)SystemParameters.PrimaryScreenHeight).Position.X;
            float posY = joinCursorHand.ScaleTo((int)SystemParameters.PrimaryScreenWidth, (int)SystemParameters.PrimaryScreenHeight).Position.Y;
            Joint scaledCursorJoint = new Joint
            {
                TrackingState = JointTrackingState.Tracked,
                Position = new Microsoft.Research.Kinect.Nui.Vector
                {
                    X = posX,
                    Y = posY,
                    Z = joinCursorHand.Position.Z
                }
            };
            OnButtonLocationChanged(kinectButton, buttons, (int)scaledCursorJoint.Position.X, (int)scaledCursorJoint.Position.Y);
        }

        void nui_DepthFrameReady(object sender, ImageFrameReadyEventArgs e)
        {
            PlanarImage Image = e.ImageFrame.Image;
            byte[] convertedDepthFrame = convertDepthFrame(Image.Bits);

            depthImage.Source = BitmapSource.Create(
                Image.Width, Image.Height, 96, 96, PixelFormats.Bgr32, null, convertedDepthFrame, Image.Width * 4);

            ++totalFrames;

            DateTime cur = DateTime.Now;
            if (cur.Subtract(lastTime) > TimeSpan.FromSeconds(1))
            {
                int frameDiff = totalFrames - lastFrames;
                lastFrames = totalFrames;
                lastTime = cur;
            }
        }

        // Converts a 16-bit grayscale depth frame which includes player indexes into a 32-bit frame
        // that displays different players in different colors
        byte[] convertDepthFrame(byte[] depthFrame16)
        {
            for (int i16 = 0, i32 = 0; i16 < depthFrame16.Length && i32 < depthFrame32.Length; i16 += 2, i32 += 4)
            {
                int player = depthFrame16[i16] & 0x07;
                int realDepth = (depthFrame16[i16 + 1] << 5) | (depthFrame16[i16] >> 3);
                // transform 13-bit depth information into an 8-bit intensity appropriate
                // for display (we disregard information in most significant bit)
                byte intensity = (byte)(255 - (255 * realDepth / 0x0fff));

                depthFrame32[i32 + RED_IDX] = 0;
                depthFrame32[i32 + GREEN_IDX] = 0;
                depthFrame32[i32 + BLUE_IDX] = 0;

                // choose different display colors based on player
                switch (player)
                {
                    case 0:
                        depthFrame32[i32 + RED_IDX] = (byte)(intensity / 2);
                        depthFrame32[i32 + GREEN_IDX] = (byte)(intensity / 2);
                        depthFrame32[i32 + BLUE_IDX] = (byte)(intensity / 2);
                        break;
                    case 1:
                        depthFrame32[i32 + RED_IDX] = intensity;
                        break;
                    case 2:
                        depthFrame32[i32 + GREEN_IDX] = intensity;
                        break;
                    case 3:
                        depthFrame32[i32 + RED_IDX] = (byte)(intensity / 4);
                        depthFrame32[i32 + GREEN_IDX] = (byte)(intensity);
                        depthFrame32[i32 + BLUE_IDX] = (byte)(intensity);
                        break;
                    case 4:
                        depthFrame32[i32 + RED_IDX] = (byte)(intensity);
                        depthFrame32[i32 + GREEN_IDX] = (byte)(intensity);
                        depthFrame32[i32 + BLUE_IDX] = (byte)(intensity / 4);
                        break;
                    case 5:
                        depthFrame32[i32 + RED_IDX] = (byte)(intensity);
                        depthFrame32[i32 + GREEN_IDX] = (byte)(intensity / 4);
                        depthFrame32[i32 + BLUE_IDX] = (byte)(intensity);
                        break;
                    case 6:
                        depthFrame32[i32 + RED_IDX] = (byte)(intensity / 2);
                        depthFrame32[i32 + GREEN_IDX] = (byte)(intensity / 2);
                        depthFrame32[i32 + BLUE_IDX] = (byte)(intensity);
                        break;
                    case 7:
                        depthFrame32[i32 + RED_IDX] = (byte)(255 - intensity);
                        depthFrame32[i32 + GREEN_IDX] = (byte)(255 - intensity);
                        depthFrame32[i32 + BLUE_IDX] = (byte)(255 - intensity);
                        break;
                }
            }
            return depthFrame32;
        }

        private void OnButtonLocationChanged(HoverButton hand, List<Button> buttons, int X, int Y)
        {
            if (IsButtonOverObject(hand, buttons))
            {
                hand.Hovering();
            }
            else
            {
                hand.Release();
            }
            Canvas.SetLeft(hand, X - (hand.ActualWidth / 2));
            Canvas.SetTop(hand, Y - (hand.ActualHeight / 2));
        }

        private static bool IsButtonOverObject(FrameworkElement hand, List<Button> buttons)
        {
            if (_isClosing || !Window.GetWindow(hand).IsActive) return false;

            // get the location of the top left of the hand and then use it to find the middle of the hand
            var handTopLeft = new Point(Canvas.GetTop(hand), Canvas.GetLeft(hand));
            _handLeft = handTopLeft.X + (hand.ActualWidth / 2);
            _handTop = handTopLeft.Y + (hand.ActualHeight / 2);

            foreach (Button target in buttons)
            {
                Point targetTopLeft = target.PointToScreen(new Point());
                if (_handTop > targetTopLeft.X
                    && _handTop < targetTopLeft.X + target.ActualWidth
                    && _handLeft > targetTopLeft.Y
                    && _handLeft < targetTopLeft.Y + target.ActualHeight)
                {
                    _selectedButton = target;
                    return true;
                }
            }
            return false;
        }

        

        void nui_VideoFrameReady(object sender, ImageFrameReadyEventArgs e)
        {
            //pull out the video frame from the eventargs and load it into our image object
            PlanarImage image = e.ImageFrame.Image;
            BitmapSource source = BitmapSource.Create(image.Width, image.Height, 96, 96,
                PixelFormats.Bgr32, null, image.Bits, image.Width * image.BytesPerPixel);
            colorImage.Source = source;
        }

        //private void InitializeRFID()
        //{
        //    try
        //    {
        //        //Initialize RFID reader
        //        rfid = new RFID();
        //        rfid.Attach += new AttachEventHandler(rfid_Attach);
        //        rfid.Detach += new DetachEventHandler(rfid_Detach);
        //        rfid.Error += new ErrorEventHandler(rfid_Error);
        //        rfid.Tag += new TagEventHandler(rfid_Tag);
        //        rfid.TagLost += new TagEventHandler(rfid_TagLost);
        //        rfid.open();
        //        Console.WriteLine("Attaching rfid reader...");
        //        rfid.waitForAttachment(2000);
        //        rfid.Antenna = true;
        //        rfid.LED = true;
        //        rfid.LED = false;
        //        Console.WriteLine("RFID reader is ready.");
        //    }
        //    catch (COMException comex)
        //    {
        //        LogEvent(comex.Message);
        //        LogEvent(comex.StackTrace);
        //        Actions.GeneralActions.Speak("There was a problem initializing the R F I D reader that I could not catch properly. Please check into it.");
        //        Actions.GeneralActions.Speak("The reader will be down until the problem is resolved. Thank you.");
        //    }
        //    catch (Exception ex)
        //    {
        //        Actions.GeneralActions.Speak("There was a problem initializing the R F I D reader. Please check the log for more information.");
        //        LogEvent(ex.Message);
        //        LogEvent(ex.StackTrace);
        //    }
        //}

        private void InitializeKinect()
        {
            try
            {
                //Initialize Kinect
                nui = Runtime.Kinects[0];// new Runtime();
                nui.Initialize(RuntimeOptions.UseDepthAndPlayerIndex | RuntimeOptions.UseDepth | RuntimeOptions.UseColor | RuntimeOptions.UseSkeletalTracking);
                nuiInitialized = true; nui.SkeletonEngine.TransformSmooth = true;
                nui.SkeletonEngine.SmoothParameters = new TransformSmoothParameters
                {
                    Smoothing = 0.75f,
                    Correction = 0.0f,
                    Prediction = 0.0f,
                    JitterRadius = 0.05f,
                    MaxDeviationRadius = 0.04f
                };
                nui.VideoStream.Open(ImageStreamType.Video, 2, ImageResolution.Resolution640x480, ImageType.Color);
                nui.DepthStream.Open(ImageStreamType.Depth, 2, ImageResolution.Resolution320x240, ImageType.DepthAndPlayerIndex);
            }
            catch (Exception ex)
            {
                Actions.GeneralActions.Speak("There was a problem initializing the Kin necked device. Please check the log for more information.");
                LogEvent(ex.Message);
                LogEvent(ex.StackTrace);
            }
        }

        void sp_tkr_SpeechContextChanged(object sender, SpeechTracker.SpeechContextChangedArgs e)
        {
            txtBlkContextLoaded.Text = e.context.ToString();
            txtBlkContextHistCnt.Text = _context.ContextHistory.Count.ToString();
        }

        void sp_tkr_SpeechDetected(object sender, SpeechTracker.IntentionArgs e)
        {
            //here we handle the event thrown by the SpeechTracker when it hears something in one of the grammars
            switch (e.context)
            {
                case SpeechTracker.Context.General:
                    switch (e.general)
                    {
                        case SpeechTracker.General.Shutdown:
                            this.Close();
                            break;
                        case SpeechTracker.General.Time:
                            GeneralActions.SayTime();
                            break;
                        case SpeechTracker.General.Log:
                            break;
                        case SpeechTracker.General.Pulse:
                            SpeakResponse();
                            break;
                        case SpeechTracker.General.Schedule:
                            break;
                        case SpeechTracker.General.ColorViewer:
                            trackHands = false;// if we are not on the tab with buttons, we shouldn't track the hands
                            this.tabItemRGB.Focus();
                            break;
                        case SpeechTracker.General.DepthViewer:
                            trackHands = false;// if we are not on the tab with buttons, we shouldn't track the hands
                            this.tabItemDepth.Focus();
                            break;
                        case SpeechTracker.General.SkeletonViewer:
                            trackHands = false;// if we are not on the tab with buttons, we shouldn't track the hands
                            this.tabItemSkeleton.Focus();
                            break;
                        case SpeechTracker.General.Buttons:
                            trackHands = true;// if we are on the tab with buttons, we should assume that the user wants to interact with the buttons
                            this.tabItemButtons.Focus();
                            break;
                        case SpeechTracker.General.Affirmation:
                            //todo: implement some intelligence to handle positive feedback
                            GeneralActions.Speak("Thank you.");
                            break;
                        case SpeechTracker.General.Thanks:
                            GeneralActions.Speak("You are welcome, " + _context.EndUser.Preferred_name);
                            break;
                        case SpeechTracker.General.Negative:
                            //todo: implement some intelligence to handle negative feedback
                            GeneralActions.Speak("I am sorry.");
                            break;
                        case SpeechTracker.General.TrackOff:
                            GeneralActions.SpeakAsync("Tracking off");
                            trackHands = false;
                            break;
                        case SpeechTracker.General.TrackOn:
                            trackHands = true;
                            GeneralActions.SpeakAsync("Tracking on");
                            tabItemButtons.Focus();
                            break;
                        case SpeechTracker.General.TranslationView:
                            tabItemTranslation.Focus();
                            break;
                        case SpeechTracker.General.NewsView:
                            tabItemNews.Focus();
                            break;
                        default:
                            break;
                    }
                    break;
                case SpeechTracker.Context.Entertain:
                    switch (e.entertain)
                    {
                        case SpeechTracker.Entertain.Music:
                            break;
                        case SpeechTracker.Entertain.Video:
                            break;
                    }
                    break;
                case SpeechTracker.Context.Meeting:
                    switch (e.meeting)
                    {
                        case SpeechTracker.Meeting.CaptureAudio:
                            break;
                        case SpeechTracker.Meeting.CaptureVideo:
                            break;
                        case SpeechTracker.Meeting.Snapshot:
                            BitmapSource image = (BitmapSource)colorImage.Source;
                            image.Save(@"G:\capture.jpg", ImageFormat.Jpeg);
                            break;
                    }
                    break;
                case SpeechTracker.Context.Office:
                    switch (e.office)
                    {
                        case SpeechTracker.Office.Excel:
                            //Process.Start("EXCEL");
                            _context.ButtonActionEvaluator.ActionPending = PendingAction.Excel;

                            break;
                        case SpeechTracker.Office.Outlook:
                            Process.Start("IEXPLORE", "http://www.live.com");
                            break;
                        case SpeechTracker.Office.PowerPoint:
                            if (Process.GetProcesses().Any(prc => prc.ProcessName.Contains("POWERPNT")))
                            {
                                OfficeActions.GivePwrPntFocus();
                            }
                            else
                            {
                                Process.Start("POWERPNT");
                            }
                            break;
                        case SpeechTracker.Office.OpenAPresentation:
                            _context.ButtonActionEvaluator.ActionPending = PendingAction.Powerpoint;
                            break;
                        case SpeechTracker.Office.StartPresentation:
                            if (!Process.GetProcesses().Any(prc => prc.ProcessName.Contains("POWERPNT")))
                            {
                                GeneralActions.Speak("I am sorry, but you have not opened a brief to present. I do not know how to assist you.");
                                return;
                            }
                            OfficeActions.GivePwrPntFocus();
                            System.Windows.Forms.SendKeys.SendWait("{F5}");
                            break;
                        case SpeechTracker.Office.PPTNextSlide:
                            OfficeActions.GivePwrPntFocus();
                            System.Windows.Forms.SendKeys.SendWait("{RIGHT}");
                            break;
                        case SpeechTracker.Office.PPTPreviousSlide:
                            OfficeActions.GivePwrPntFocus();
                            System.Windows.Forms.SendKeys.SendWait("{LEFT}");
                            break;
                        case SpeechTracker.Office.Word:
                            break;
                        case SpeechTracker.Office.OpenADocument:
                            _context.ButtonActionEvaluator.ActionPending = PendingAction.Word;
                            break;
                        case SpeechTracker.Office.OpenAProjectPlan:
                            _context.ButtonActionEvaluator.ActionPending = PendingAction.ProjectPlan;
                            break;
                    }
                    break;
                case SpeechTracker.Context.Operations:
                    switch (e.operations)
                    {
                        case SpeechTracker.Operations.CameraDown:
                            CameraDown();
                            break;
                        case SpeechTracker.Operations.CameraUp:
                            CameraUp();
                            break;
                        case SpeechTracker.Operations.CameraCenter:
                            CameraCenter();
                            break;
                        case SpeechTracker.Operations.SkeletonTracker:
                            break;
                        case SpeechTracker.Operations.FineControl:
                            cameraControl = CameraControl.Fine;
                            break;
                        case SpeechTracker.Operations.CourseControl:
                            cameraControl = CameraControl.Course;
                            break;
                    }
                    break;
                case SpeechTracker.Context.Research:
                    switch (e.research)
                    {
                        case SpeechTracker.Research.Lookup:
                            break;
                        case SpeechTracker.Research.News:
                            tabItemNews.Focus();
                            txtBlkNews.Text = NewsActions.GetTheNews();
                            break;
                        case SpeechTracker.Research.Spelling:
                            break;
                    }
                    break;
                case SpeechTracker.Context.Translate:
                    switch (e.translate)
                    {
                        case SpeechTracker.Translate.ArabicEnglish:
                            tabItemTranslation.Focus();
                            txtBlkTranslation.Text = BingTranslationActions.TranslationRequest("ar", "en", "مرحبا، وآمل كنت تواجه يوم جميل.");
                            break;
                        case SpeechTracker.Translate.EnglishArabic:
                            tabItemTranslation.Focus();
                            txtBlkTranslation.Text = BingTranslationActions.TranslationRequest("en", "ar", "hello, I hope you are having a nice day.");
                            break;
                        case SpeechTracker.Translate.EnglishSpanish:
                            tabItemTranslation.Focus();
                            txtBlkTranslation.Text = BingTranslationActions.TranslationRequest("en", "es", "hello, I hope you are having a nice day.");
                            break;
                        case SpeechTracker.Translate.EnglishTurkish:
                            tabItemTranslation.Focus();
                            txtBlkTranslation.Text = BingTranslationActions.TranslationRequest("en", "tr", "hello, I hope you are having a nice day.");
                            break;
                        case SpeechTracker.Translate.SpanishEnglish:
                            tabItemTranslation.Focus();
                            txtBlkTranslation.Text = BingTranslationActions.TranslationRequest("es", "en", "Hola, espero que está teniendo un buen día.");
                            break;
                        case SpeechTracker.Translate.TurkishEnglish:
                            tabItemTranslation.Focus();
                            txtBlkTranslation.Text = BingTranslationActions.TranslationRequest("tr", "en", "Merhaba, güzel bir gün yaşıyorsanız umuyoruz.");
                            break;
                    }
                    break;
            }
            txtBlkHeard.Text = e.phrase;
        }

       
        private void CameraUp()
        { 
            int x = nui.NuiCamera.ElevationAngle;
            x = x + GetCameraDeltaFactor();
            if (x > 27)
                x = 27;
            nui.NuiCamera.ElevationAngle = x;
        }

        private int GetCameraDeltaFactor()
        {
            int delta;
            if (cameraControl == CameraControl.Course)
                delta = 5;
            else
                delta = 1;
            return delta;
        }

        private void CameraDown()
        {
            int x = nui.NuiCamera.ElevationAngle;
            x = x - GetCameraDeltaFactor();
            if (x < -27)
                x = -27;
            nui.NuiCamera.ElevationAngle = x;
        }

        private void CameraCenter()
        {
            nui.NuiCamera.ElevationAngle = 0;
        }

        private void InitializeButtons()
        {
            buttons = new List<Button>();
            buttons.Add(button1);
            buttons.Add(button2);
            buttons.Add(button3);
            buttons.Add(button4);
            buttons.Add(button5);
            buttons.Add(button6);
            buttons.Add(button7);
            buttons.Add(button8);
        }

        #region Button events

        void button1_Click(object sender, RoutedEventArgs e)
        {
            _context.ButtonActionEvaluator.SelectedButton = ButtonSelected.Button1;
            //GeneralActions.Speak("You clicked button number one.");
        }

        void button2_Click(object sender, RoutedEventArgs e)
        {
            _context.ButtonActionEvaluator.SelectedButton = ButtonSelected.Button2;
            //GeneralActions.Speak("You clicked button number two.");
        }

        void button3_Click(object sender, RoutedEventArgs e)
        {
            _context.ButtonActionEvaluator.SelectedButton = ButtonSelected.Button3;
            //GeneralActions.Speak("You clicked button number three.");
        }

        void button4_Click(object sender, RoutedEventArgs e)
        {
            _context.ButtonActionEvaluator.SelectedButton = ButtonSelected.Button4;
            //GeneralActions.Speak("You clicked button number four.");
        }

        void button5_Click(object sender, RoutedEventArgs e)
        {
            _context.ButtonActionEvaluator.SelectedButton = ButtonSelected.Button5;
            //GeneralActions.Speak("You clicked button number five.");
        }

        void button6_Click(object sender, RoutedEventArgs e)
        {
            _context.ButtonActionEvaluator.SelectedButton = ButtonSelected.Button6;
            //GeneralActions.Speak("You clicked button number six.");
        }

        void button7_Click(object sender, RoutedEventArgs e)
        {
            _context.ButtonActionEvaluator.SelectedButton = ButtonSelected.Button7;
            //GeneralActions.Speak("You clicked button number seven.");
        }

        void button8_Click(object sender, RoutedEventArgs e)
        {
            _context.ButtonActionEvaluator.SelectedButton = ButtonSelected.Button8;
            //GeneralActions.Speak("You clicked button number eight.");
        }

        #endregion

        private void UninitializeNui()
        {
            if ((nui != null) && (nuiInitialized))
            {
                nui.Uninitialize();
                nuiInitialized = false;
            }
        }

        private void LoadRfidAccounts()
        {
            //I used an RFID reader in the project for GMU.
            //It gave me a convenient way to identify users.
            //I have removed the need for it in this code, since most readers won't have a Phidgets RFID reader to play with.
            knownPeople = new Dictionary<string, Person>();
            knownPeople.Add("110033e117", new Person("110033e117", Person.Gender.Male, "Leland Holmquest", "Leland"));
            knownPeople.Add("110033d0bd", new Person("110033d0bd", Person.Gender.Female, "Jane Doe", "Jane"));
            knownPeople.Add("01068df9e2", new Person("01068df9e2", Person.Gender.Male, "User One", "One"));
            knownPeople.Add("0107ee5ccd", new Person("0107ee5ccd", Person.Gender.Male, "User two", "Two"));
            knownPeople.Add("01068dfb65", new Person("01068dfb65", Person.Gender.Female, "Kenzy Holmquest", "Kenzy"));
            knownPeople.Add("0107ee8771", new Person("0107ee8771", Person.Gender.Female, "Shahrazad Holmquest", "Sherrie"));
            knownPeople.Add("No tag in view", new Person("No tag in view", Person.Gender.Unknown, "no person", "no person"));
        }

        static private void LogEvent(string text)
        {
            string source = "Project Lily";

            if (!EventLog.SourceExists(source))
                EventLog.CreateEventSource(source, "Application");
            EventLog.WriteEntry(source, text);
        }

        /// <summary>
        /// This method is used by Dispatcher to update the text in the txtBlkRFIDTag textblock.
        /// This is necessary because the RFID library is operating on a different thread and cannot update the UI thread.
        /// </summary>
        /// <param name="text">(string) that represents the desired text in txtBlkRFIDTag Textblock.</param>
        private void SetRFIDTagText(string text)
        {
            if (text == "No tag in view")
            {
                txtBlkRFIDTag.Text = "No tag currently attached.";
                return;
            }
            if(!knownPeople.ContainsKey(text))
            {
                //the rfid read is not known to the system =>
                // advise user and bail from method
                GeneralActions.SpeakAsync("An R F I D was read that I am not aware of. If you want me to make use of this I D, please give me the knowledge. Thank you.");
                txtBlkRFIDTag.Text = text;
                return;
            }
            string txt = "";
            Person acnt = knownPeople[text];
            txt = acnt.Full_name + ", (preferred name: " + acnt.Preferred_name + ") logged in.";
            txtBlkRFIDTag.Text = txt;
            if (!acnt.greetedToday())
            {
                //person has not been greeted by system yet - greet
                GeneralActions.Greet(acnt.Preferred_name);
                acnt.LastGreeted = DateTime.Now;
                // we need to update this account => remove and re-add
                knownPeople.Remove(acnt.Rfid);
                knownPeople.Add(acnt.Rfid, acnt);
            }
        }

        /// <summary>
        /// This method is used by Dispatcher to insert an RFID Tag into the history.
        /// </summary>
        /// <param name="tag">(string) that represents the RFID Tag recognized by the system.</param>
        private void QueueRFIDHistory(string tag)
        {
            rfidHistory.Enqueue(tag);
        }

        /// <summary>
        /// This method gives a verbal response to a pulse request from the user.
        /// Some reponses use a personal identifier like "Yes, [name]".
        /// To make it more humanistic, we randomly substitute the user's name with a gender appropriate title.
        /// </summary>
        private void SpeakResponse()
        {
            string ac = "";
            Random rand1 = new Random();
            int x = rand1.Next(10);
            if ((x % 2) == 0)
            {
                ac = _context.EndUser.Preferred_name;
            }
            else
            {
                switch (_context.EndUser.PersonGender)
                {
                    case Person.Gender.Male:
                        ac = "sir";
                        break;
                    case Person.Gender.Female:
                        ac = "ma'am";
                        break;
                    default:
                        ac = "boss";
                        break;
                }
            }
            Random rand = new Random();
            int r = rand.Next(pulseResponses.Count<string>());
            if (pulseResponses[r].Contains("{0}"))
            {
                GeneralActions.Speak(string.Format(pulseResponses[r], ac));
            }
            else
            {
                GeneralActions.Speak(pulseResponses[r]);
            }
        }

        private void chkAEC_Click(object sender, RoutedEventArgs e)
        {
                            
        }

        #region RFID Events

        //void rfid_TagLost(object sender, TagEventArgs e)
        //{
        //    Dispatcher.Invoke(DispatcherPriority.Normal,
        //                         new Action<string>(SetRFIDTagText),
        //                         "No tag in view");
        //    ((RFID)sender).LED = false;
        //    currentRFIDTag = string.Empty;
        //}

        //void rfid_Tag(object sender, TagEventArgs e)
        //{
        //    Dispatcher.Invoke(DispatcherPriority.Normal,
        //                      new Action<string>(SetRFIDTagText),
        //                      e.Tag);
        //    Dispatcher.Invoke(DispatcherPriority.Normal,
        //                      new Action<string>(QueueRFIDHistory),
        //                      e.Tag);
        //    ((RFID)sender).LED = true;
        //    currentRFIDTag = e.Tag;
        //}

        //void rfid_Error(object sender, ErrorEventArgs e)
        //{
        //    LogEvent("RFID error: " + e.Description);
        //    LogEvent("RFID error: " + e.exception.Message + " STACK: " + e.exception.StackTrace);
        //}

        //void rfid_Detach(object sender, DetachEventArgs e)
        //{
        //    Console.WriteLine("RFID reader detached");
        //}

        //void rfid_Attach(object sender, AttachEventArgs e)
        //{
        //    Console.WriteLine("RFID reader attached");
        //}

        #endregion

        #region WPF Form Events

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
        }

        private void Window_GotKeyboardFocus(object sender, KeyboardFocusChangedEventArgs e)
        {

        }

        private void Window_Activated(object sender, EventArgs e)
        {

        }

        private void Window_ContentRendered(object sender, EventArgs e)
        {
            Actions.GeneralActions.Greet(_context.EndUser.Preferred_name);// Environment.UserName);
            Actions.GeneralActions.SayTime();
            //if (rfid.Attached)
            //{
            //    Actions.GeneralActions.Speak("The R F I D reader is ready.");
            //}
            //else
            //{
            //    Actions.GeneralActions.Speak("There is a problem with the R F I D reader.");
            //}
            if (nuiInitialized)
            {
                GeneralActions.Speak("The kin necked unit is ready.");
            }
            else
            {
                GeneralActions.Speak("The kin necked unit is not available. Without it, I am blind and deaf, and therefore not truly functional. Please look into correcting the problem.");
            }

            GeneralActions.Speak("I am Lily. I am on line and ready to serve.");
        }

        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            GeneralActions.Speak("I am shutting down this service per your direction, " + _context.EndUser.Preferred_name);
            //window is closing => shutdown Kinect and RFID reader
            //Shutdown RFID reader
            //rfid.LED = false;//shut off LED
            //rfid.close();
            //rfid = null;
            //Actions.GeneralActions.Speak("The R F I D reader has shut down.");
            //Console.WriteLine("RFID reader is shutdown.");
            if (sp_tkr != null)
            {
                sp_tkr.KeepListening = false;
            }

            //Shutdown Kinect
            UninitializeNui();
            Console.WriteLine("Kinect unit released.");
            Actions.GeneralActions.Speak("The kin necked unit has shut down.");


            GeneralActions.Speak("The service is shutting down.");
            Actions.GeneralActions.SayTime();
            Actions.GeneralActions.Goodbye(Environment.UserName);
        }

        #endregion

        
        
        #endregion

        #region Public Methods

        #endregion

    }
}
